import asyncio
from dataclasses import dataclass
from itertools import zip_longest
import math
import time
from typing import Tuple
from textual import events
from textual.app import App, ComposeResult, RenderResult
from textual.reactive import reactive
from textual.widgets import Header, Footer, Static
from textual.widget import Widget
from textual.binding import Binding


from rich.console import Console, ConsoleOptions
from rich.segment import Segment
from rich.style import Style
from rich.color import Color
from rich.measure import Measurement

from fractions import Fraction as F

from PIL import Image


import numpy as np

from textual_imgview.img import ImageSection


class ImageViewer(Widget):
    ZOOM_RATE = 0.8
    MOVEMENT_PERIOD = 0.25

    # BINDINGS = [
    #     Binding("r", "zoom(-1)", "Zoom In", show=True, priority=True),
    #     ("minus", "zoom(1)", "Zoom Out")
    # ]

    # (w, h) size of this widget
    # dimensions = reactive((0, 0), layout=True)

    # # Image-space position of widget origin
    # origin_position = reactive((0, 0))

    # # Representation of image at current zoom level
    # zoom = reactive(0)

    # def action_zoom(self, amount: int):
    #     self.zoom += amount
    #     print('zoomed')

    def __init__(
        self,
        image: np.ndarray,
        # *children: Widget,
        # name: str | None = None,
        # id: str | None = None,
        # classes: str | None = None,
    ) -> None:
        if not isinstance(image, np.ndarray):
            raise TypeError(
                "Expected image with type np.ndarray, but received "
                f"{type(image.name)} instead."
            )

        if len(image.shape) == 2:
            image = image[:, :, np.newaxis]
        elif len(image.shape) != 3:
            raise ValueError(
                "Expected image to have 3 dimensions (H x W x C), but received "
                f"{len(image.shape)} dimensions instead."
            )

        if image.shape[-1] == 1:
            image = image.repeat(3, -1)
        elif image.shape[-1] != 3:
            raise ValueError(
                "Expected image to have 1 or 3 channels, but received "
                f"{image.shape[-1]} channels instead."
            )

        if (
            np.issubdtype(image.dtype, np.floating)
            and np.min(image) >= 0
            and np.max(image) <= 1
        ):
            image = image * 256

        self.image = Image.fromarray(image.astype(np.uint8))

        # Whether or not the left mouse buttom is currently pressed down
        self.mouse_down = False

        # Terminal position of mouse
        self.mouse_position = None

        self.image_section = Static(ImageSection(self.image), id="img")
        super().__init__(self.image_section, Static("hello"))

        # self.last_fast_refresh = 0

    def on_show(self):
        pass
        # Basically on_mount but we know the size
        # w, h = self.size.width, self.size.height  # Initially (0, 0) for now
        # img_w, img_h = self.image.size
        # zoom_w = math.log(w / img_w, self.ZOOM_RATE)
        # zoom_h = math.log((h * 2) / img_h, self.ZOOM_RATE)

        # self.zoom = max(0, max(zoom_w, zoom_h))

        # self.watch_zoom(self.zoom)

        # img_h, img_w, _ = self.zoomed_image.shape
        # self.dimensions = (w, h)
        # self.origin_position = (-round((w - img_w) / 2), -round(h - img_h / 2))

    # async def on_mouse_scroll_down(self, event: events.MouseScrollDown):
    #     self.zoom += 1

    # async def on_mouse_scroll_up(self, event: events.MouseScrollDown):
    #     self.zoom -= 1

    def on_mouse_down(self, event: events.MouseDown):
        print("mouse down")
        self.mouse_down = True
        self.capture_mouse(capture=True)
        print(self.image_section.region)

    def on_mouse_up(self, event: events.MouseDown):
        self.mouse_down = False
        self.capture_mouse(capture=False)

    def on_mouse_move(self, event: events.MouseMove):
        # self.mouse_position = event.x, event.y
        print("mouse move")

        if self.mouse_down and (event.delta_x != 0 or event.delta_y != 0):
            print("updating")
            # origin_x, origin_y = self.origin_position

            # self.origin_position = (
            #     origin_x - event.delta_x,
            #     origin_y - 2 * event.delta_y,
            # )
            offset = self.image_section.styles.offset
            self.image_section.styles.offset = (
                offset.x.value + event.delta_x,
                offset.y.value + event.delta_y,
            )

    # def on_resize(self, event: events.Resize):
    #     old_w, old_h = self.dimensions
    #     new_w, new_h = event.size.width, event.size.height

    #     origin_x, origin_y = self.origin_position

    #     if new_w != old_w:
    #         delta_w = new_w - old_w

    #         if delta_w % 2 == 0:
    #             origin_x = origin_x - delta_w // 2
    #         else:
    #             # If we're 1 by 1 resizing, this keeps things even
    #             op = math.floor if new_w % 2 == 0 else math.ceil
    #             origin_x = op(origin_x - delta_w / 2)

    #     if new_h != old_h:
    #         delta_h = new_h - old_h
    #         origin_y -= delta_h

    #     self.origin_position = (origin_x, origin_y)
    #     self.dimensions = new_w, new_h

    # def validate_origin_position(
    #     self, origin_position: Tuple[int, int]
    # ) -> Tuple[int, int]:
    #     origin_x, origin_y = origin_position
    #     w, h = self.dimensions
    #     img_h, img_w, _ = self.zoomed_image.shape

    #     if origin_x <= -w + 1:
    #         origin_x = -w + 1

    #     if origin_x >= img_w - 1:
    #         origin_x = img_w - 1

    #     if origin_y <= -2 * h + 1:
    #         origin_y = -2 * h + 1

    #     if origin_y >= img_h - 1:
    #         origin_y = img_h - 1

    #     return origin_x, origin_y

    # def validate_zoom(self, zoom: int):
    #     if zoom <= 0:
    #         return 0

    #     if zoom > self.zoom and min(self.zoomed_image.shape[:-1]) <= 8:
    #         return self.zoom

    #     return zoom

    # def watch_zoom(self, zoom: int):
    #     multiplier = self.ZOOM_RATE**zoom

    #     w, h = self.image.size
    #     new_w, new_h = round(w * multiplier), round(h * multiplier)

    #     if hasattr(self, "zoomed_image"):
    #         old_h, old_w, _ = self.zoomed_image.shape

    #     self.zoomed_image = np.asarray(self.image.resize((new_w, new_h)))

    #     # If mouse position is available, zoom in at mouse position
    #     if self.mouse_position is not None:
    #         origin_x, origin_y = self.origin_position
    #         mouse_term_x, mouse_term_y = self.mouse_position
    #         old_mouse_x = origin_x + mouse_term_x
    #         old_mouse_y = origin_y + 2 * mouse_term_y

    #         multiplier_x = new_w / old_w
    #         multiplier_y = new_h / old_h

    #         new_mouse_x = old_mouse_x * multiplier_x
    #         new_mouse_y = old_mouse_y * multiplier_y

    #         self.origin_position = (
    #             origin_x + round(new_mouse_x - old_mouse_x),
    #             origin_y + round(new_mouse_y - old_mouse_y),
    #         )

    # def fast_refresh(self):
    #     self.last_fast_refresh = time.time()

    #     def timed_refresh():
    #         if time.time() - self.last_fast_refresh > self.MOVEMENT_PERIOD:
    #             self.refresh()

    #     self.set_timer(self.MOVEMENT_PERIOD, timed_refresh)

    # def render(self) -> RenderResult:
    #     print("rendered")
    #     return Static(self.image_section)
    # IDEA: THROW OUT IMG.PY, AND DO IT ALL IN HERE


class ImageViewerApp(App):
    """"""

    CSS_PATH = "styles.css"

    def compose(self) -> ComposeResult:
        """Create child widgets for the app."""

        l = 64
        image = np.arange(l * l).reshape(l, l) / (l * l)

        # import imageio

        # image = imageio.imread("./img.jpg")

        # yield Header()
        yield ImageViewer(image)
        yield Footer()


if __name__ == "__main__":

    app = ImageViewerApp()
    app.run()
